#!/usr/bin/env python

# Copyright 2020 Unlimited Robotics

import argparse
import os
import sys
import platform

from rayasdk.connect import URConnect

from rayasdk.logger import log, LogLevels, log_error, log_verbose, set_logger_level
from rayasdk.scanner import URScanner
from rayasdk.initializer import URInitializer
from rayasdk.runner import URRunner
from rayasdk.simulator import URSimulator
from rayasdk.killer import URKiller
from rayasdk.constants import *

from rayasdk.container_handlers.docker_handler import check_container

class URSDK:
    def __init__(self):
        self.command_objects = []
        self.command_objects.append(URInitializer())
        self.command_objects.append(URScanner())
        self.command_objects.append(URConnect())
        self.command_objects.append(URRunner())
        self.command_objects.append(URSimulator())
        self.command_objects.append(URKiller())
        self.init_parser()
    
    def init_parser(self):
        # Init top parser
        self.argparser = argparse.ArgumentParser(description='Unlimited Robotics SDK')
        group = self.argparser.add_mutually_exclusive_group()
        group.add_argument("-v", "--verbose", action="store_true", help="increase output verbosity.")
        group.add_argument("-q", "--quiet", action="store_true", help="don't print on stdout.")
        subparsers = self.argparser.add_subparsers(help='SDK Command', dest='command')
        subparsers.required = True
        # Init subparsers
        for obj in self.command_objects:
            obj.init_parser(subparsers)
        # Parse arguments
        self.args, self.unknownargs = self.argparser.parse_known_args()
        # Setup Logger
        set_logger_level(self.args.verbose, self.args.quiet)

    def init_ursdk_folder(self):
        if not os.path.exists(URSDK_TEMP_PATH):
            log_verbose(f'Folder {URSDK_TEMP_PATH} does not exists, creating it.')
            try:
                os.mkdir(URSDK_TEMP_PATH)
            except OSError as exc:
                log_error(f'Folder {URSDK_TEMP_PATH} could not be created.')
                return False
            return True
        else:
            log_verbose(f'Folder {URSDK_TEMP_PATH} found.')
            return True

    def run(self):
        if not self.init_ursdk_folder():
            return False
        if not check_container():
            return False
        for obj in self.command_objects:
            if self.args.command == type(obj).COMMAND:
                return obj.run(self.args, self.unknownargs)

def main():
    if platform.system() not in ['Linux', 'Windows']:
        print(f'Platform \'{platform.system()}\' not supported.')
        exit(1)
    ursdk = URSDK()
    ret = ursdk.run()
    if not ret:
        log_verbose('Finished with error.')
        sys.exit(1)
    else:
        sys.exit(0)


if __name__ == "__main__":
    main()


import os

from src.app import RayaApplication
from raya.entry_point import entry_point


def main():
    app_path = os.path.dirname(os.path.realpath(__file__))
    entry_point(app_path, RayaApplication)


if __name__ == "__main__":
    main()



# Copyright 2020 Unlimited Robotics
import json
import subprocess
import sys
import os
from rayasdk.constants import *
from rayasdk.logger import *
from rayasdk.container_handlers.docker_handler import scanner, launch_app


class URRunner:

    COMMAND = 'run'


    def __init__(self):
        pass


    def init_parser(self, subparser):
        self.parser = subparser.add_parser(type(self).COMMAND,
                                           help="connect current raya project to a robot or simulator.")


    def robot_available(self, robot_id, domain, serial):
        robots_info_dict = scanner(domain, domain, silent=True)
        if robot_id not in robots_info_dict:
            return False
        if robots_info_dict[robot_id][JSON_SCAN_DDSCH] != domain:
            return False
        if robots_info_dict[robot_id][JSON_SCAN_SERIAL] != serial:
            return False
        return True


    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs
        # open exec_settings file
        try:
            with open(EXECSETTINGS_PATH, 'r', encoding='utf-8') as f:
                exec_settings = json.load(f)
        except OSError as exc:
            log_error(f'Could  not open "{EXECSETTINGS_FILE}" file, no project initialized in this folder.')
            return False

        app_id = exec_settings[JSON_EXECINFO_APPID]
        try:
            robot_id = exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBID]
            dds_domain = exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBDDSCH]
            robot_serial = exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBSERIAL]
        except KeyError:
            print("Not connected to any robot, run \'rayasdk connect\' first")
            return

        # Verify exec_settings configured, if so, check if robot is still available
        if robot_id == '':
            print("Not connected to any robot, run \'rayasdk connect\' first")
            return
        
        if not self.robot_available(robot_id, dds_domain, robot_serial):
            print(f"Robot \'{robot_id}\' with serial \'{robot_serial} is not available, run \'rayasdk connect\' again")
            return
        
        launch_app(app_id, dds_domain, self.unknownargs)



# Copyright 2020 Unlimited Robotics
import json

from rayasdk.container_handlers.docker_handler import kill_all
from rayasdk.logger import log_error, log_verbose, log_info
from rayasdk.constants import *


class URKiller:

    COMMAND = 'kill'


    def __init__(self):
        pass


    def init_parser(self, subparser):
        self.parser = subparser.add_parser(type(self).COMMAND, help="kills all the running Ra-Ya containers.")


    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs
        kill_all()
        return True



from raya.application_base import RayaApplicationBase

class RayaApplication(RayaApplicationBase):
    async def setup(self):
        # Create local attributes and variables
        self.i = 0
        self.log.info(f'Hello from setup()')

    async def loop(self):
        # Loop
        self.i += 1
        self.log.info(f'Hello from loop(), i={self.i}')
        await self.sleep(0.2)
        if self.i>=10:
            self.finish_app()

    async def finish(self):
        # Finishing instructions
        self.log.warn(f'Hello from finish()')



# Copyright 2020 Unlimited Robotics
import json

from rayasdk.container_handlers.docker_handler import scanner
from rayasdk.logger import log_error, log_verbose, log_info
from rayasdk.constants import *


class URConnect:

    COMMAND = 'connect'


    def __init__(self):
        pass


    def init_parser(self, subparser):
        self.parser = subparser.add_parser(type(self).COMMAND, help="execute current Raya project.")
        group = self.parser.add_mutually_exclusive_group(required=True)
        group.add_argument('--robot-id', help='robot identificator (from scan list)', type=str)
        group.add_argument('--simulator', help='connect the project to the simulator', action="store_true")


    def robot_available(self, robot_id, domain, serial):
        robots_info_dict = scanner(domain, domain, silent=True)
        if robot_id not in robots_info_dict:
            return False
        if robots_info_dict[robot_id][JSON_SCAN_DDSCH] != domain:
            return False
        if robots_info_dict[robot_id][JSON_SCAN_SERIAL] != serial:
            return False
        return True


    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs
        try:
            with open(EXECSETTINGS_PATH, 'r', encoding='utf-8') as f:
                exec_settings = json.load(f)
        except OSError as exc:
            log_error(f'Could not open "{EXECSETTINGS_FILE}" file, no project initialized in this folder.')
            return False
        if self.args.simulator:
            exec_settings[JSON_EXECINFO_SIM] = True
            exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBID] = ''
            exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBIP] = ''
            exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBDDSCH] = 0
        else:
            robot_id = self.args.robot_id
            try:
                with open(LAST_SCANNING_PATH, 'r', encoding='utf-8') as f:
                    scanned_robots = json.load(f)
            except OSError as exc:
                log_error(f'Could  not get scanning info, execute "ursdk scan" before this command.')
                return False
            try:
                if robot_id in scanned_robots:
                    # check robot connected to the same serial and domain
                    if self.robot_available(
                                robot_id,
                                scanned_robots[robot_id][JSON_SCAN_DDSCH],
                                scanned_robots[robot_id][JSON_SCAN_SERIAL]
                            ):
                        # if robot exists, register it.
                        exec_settings[JSON_EXECINFO_SIM] = False
                        if JSON_EXECINFO_ROBCONN not in  exec_settings:
                            exec_settings[JSON_EXECINFO_ROBCONN] = {}    
                        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBID] = \
                            robot_id
                        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBIP] = \
                            scanned_robots[robot_id][JSON_SCAN_IP]
                        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBDDSCH] = \
                            scanned_robots[robot_id][JSON_SCAN_DDSCH]
                        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBSERIAL] = \
                            scanned_robots[robot_id][JSON_SCAN_SERIAL]
                        log_info(f'You have successfully connected to {robot_id}')
                    else:
                        log_error(f'{exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBID]} is not longer available')
                else:
                    log_error(f'Robot ID "{robot_id}" not found in scan info, verify it or scan again.')
                    return False
            except KeyError as e:
                print(e)
                log_error(f'Scanning info malformed, please scan again by running "rayasdk scan".')
                return False
        try:
            with open(EXECSETTINGS_PATH, 'w', encoding='utf-8') as f:
                json.dump(exec_settings, f, ensure_ascii=False, indent=4)
        except FileNotFoundError:
            log_error(f'Could not write file "{EXECSETTINGS_FILE}".')
            return False

        log_verbose('\nCorrect!')
        return True



# Copyright 2020 Unlimited Robotics

import json
from tabulate import tabulate

from rayasdk.container_handlers.docker_handler import scanner
from rayasdk.logger import log_error, log_info, log_verbose
from rayasdk.constants import *


class URScanner:
    COMMAND = 'scan'

    def __init__(self):
        pass

    def init_parser(self, subparser):
        self.parser = subparser.add_parser(type(self).COMMAND, help="discover robots in the local network.")

    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs
        log_info('Scanning local network for robots (press Ctrl+C to stop)...', )
        robots_info_dict = scanner(1, 100)
        robots_info = []

        for robot_id in robots_info_dict:
            robots_info.append([
                robot_id,
                robots_info_dict[robot_id][JSON_SCAN_SERIAL],
                robots_info_dict[robot_id][JSON_SCAN_DDSCH],
                str(robots_info_dict[robot_id][JSON_SCAN_IP]),
            ])

        # Write json output file
        try:
            with open(LAST_SCANNING_PATH, 'w', encoding='utf-8') as f:
                json.dump(robots_info_dict, f, ensure_ascii=False, indent=4)
        except FileNotFoundError:
            log_error(f'Could not write file {LAST_SCANNING_PATH} .')
            return False

        # Screen out all the robot found in scan
        if robots_info:
            log_info('')
            log_info(tabulate(robots_info, headers=['Robot ID', 'Serial', 'DDS Domain', 'IP Address']))
        else:
            log_info('No robots found.')

        log_verbose('\nCorrect!')
        return True



# Copyright 2020 Unlimited Robotics

import tempfile
import platform
from pathlib import Path

# Json SCAN Keys
JSON_SCAN_IP = 'ip-address'
JSON_SCAN_SERIAL = 'serial'
JSON_SCAN_DDSCH = 'dds-domain'

# Json SDK Info Keys
JSON_EXECINFO_APPID = 'app-id'
JSON_EXECINFO_APPNAME = 'app-name'
JSON_EXECINFO_SIM = 'simulation'
JSON_EXECINFO_DEVMODE = 'dev-mode'
JSON_EXECINFO_ROBCONN = 'robot-connection'
JSON_EXECINFO_ROBID = 'robot-id'
JSON_EXECINFO_ROBIP = 'ip-address'
JSON_EXECINFO_ROBDDSCH = 'dds-domain'
JSON_EXECINFO_ROBSERIAL = 'robot-serial'
JSON_EXECINFO_LOG = 'logging'
JSON_EXECINFO_LOG_FILEENA = 'file_enabled'
JSON_EXECINFO_LOG_FOLDER = 'folder'

# folders
TEMP_URSDK_FOLDER = 'ursdk'
TEMPLATES_FOLDER = 'template'
DAT_FOLDER = 'dat'
DOC_FOLDER = 'doc'
LOG_FOLDER = 'log'
RES_FOLDER = 'res'
SRC_FOLDER = 'src'

# files
LAST_SCANNING_FILE = 'last_scanning.json'
ENTRYPOINT_FILE = '__main__.py'
APP_FILE = 'app.py'
MANIFEST_FILE = 'manifest.json'
EXECSETTINGS_FILE = 'exec_settings.json'

# system paths
TEMP_PATH = Path(tempfile.gettempdir())
CURRENT_PATH = Path().absolute()
URSDK_PATH = Path(__file__).parent.absolute()
UR_HOME = Path.home() / '.ur'

# derived folder paths
URSDK_TEMP_PATH = TEMP_PATH / TEMP_URSDK_FOLDER
TEMPLATES_PATH = URSDK_PATH / TEMPLATES_FOLDER
DAT_PATH = CURRENT_PATH / DAT_FOLDER
DOC_PATH = CURRENT_PATH / DOC_FOLDER
LOG_PATH = CURRENT_PATH / LOG_FOLDER
RES_PATH = CURRENT_PATH / RES_FOLDER
SRC_PATH = CURRENT_PATH / SRC_FOLDER

# derived files paths
LAST_SCANNING_PATH = URSDK_TEMP_PATH / LAST_SCANNING_FILE
ENTRYPOINT_PATH_ORIG = TEMPLATES_PATH / ENTRYPOINT_FILE
ENTRYPOINT_PATH_DEST = CURRENT_PATH / ENTRYPOINT_FILE
APP_PATH_ORIG = TEMPLATES_PATH / APP_FILE
APP_PATH_DEST = SRC_PATH / APP_FILE
MANIFEST_PATH_ORIG = TEMPLATES_PATH / MANIFEST_FILE
MANIFEST_PATH_DEST = CURRENT_PATH / MANIFEST_FILE
EXECSETTINGS_PATH = CURRENT_PATH / EXECSETTINGS_FILE

RAYASIM_UI_URL = 'https://chest-web.web.app/app/ex_ui_controller'

# Docker Environment
RAYAENV_DOCKER_IMGNAME = 'raya_os'
RAYAENV_DOCKER_VERSION = '1.0.4'
RAYAENV_DOCKER_URL = 'https://storage.googleapis.com/raya_files/Common/docker_images/raya_os_1.0.4.tar'
RAYAENV_DOCKER_SHA256 = 'd3e23ecf22f75a899a6e45bf9f3f3c2bfe6507923843df26803148ad907700d7'
RAYAENV_DOCKER_IMGFILE = f'{RAYAENV_DOCKER_IMGNAME}_{RAYAENV_DOCKER_VERSION}.tar'
RAYAENV_DOCKER_IMGPATH = URSDK_TEMP_PATH / RAYAENV_DOCKER_IMGFILE

RAYAENV_DOCKER_CONTAINERPREFIX = 'raya_os'

# Gary Simulator
GARYSIM_VERSION = '1.0.3'

if platform.system() == 'Linux':
    GARYSIM_URL = 'https://storage.googleapis.com/raya_files/Linux/raya_simulator_1.0.3_linux.tar.gz'
    GARYSIM_SHA256 = 'fc5a36929abeceffd6f0c341ad5bff942d8e94b0c8f61fc62efe17e6d4a7a225'

    SIMS_HOME = UR_HOME / 'simulator'
    SIM_PATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_linux'
    SIM_VERSION_FILE = SIM_PATH / 'VERSION.txt'
    SIM_TARPATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_linux.tar.gz'
    SIM_BINARY = SIM_PATH / 'Linux.x86_64'

elif platform.system() == 'Windows':
    GARYSIM_URL = 'https://storage.googleapis.com/raya_files/Windows/raya_simulator_1.0.3_windows.tar.gz'
    GARYSIM_SHA256 = 'a66aa2812a556322d369a1e639d101d69f06ca77470a52ab6063b6df11d3180a'

    SIMS_HOME = UR_HOME / 'simulator'
    SIM_PATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_windows'
    SIM_VERSION_FILE = SIM_PATH / 'VERSION.txt'
    SIM_TARPATH = SIMS_HOME / f'raya_simulator_{GARYSIM_VERSION}_windows.tar.gz'
    SIM_BINARY = SIM_PATH / 'Windows.exe'

# TODO: MAC version

# https://storage.googleapis.com/raya_files/Mac/raya_simulator_1.0.2_mac.tar.gz
# 


# Copyright 2020 Unlimited Robotics

import os
import json
from shutil import copyfile

from rayasdk.logger import log_error, log_verbose
from rayasdk.constants import *


class URInitializer:
    COMMAND = 'init'

    def __init__(self):
        pass

    def init_parser(self, subparser):
        self.parser = subparser.add_parser(type(self).COMMAND, help="initialize Raya project in current folder.")
        self.parser.add_argument('--app-id', help='application unique identificator', required=True, type=str)
        self.parser.add_argument('--app-name', help='application name', type=str)

    def create_files_tree(self):
        try:
            os.mkdir(DAT_PATH)
            os.mkdir(DOC_PATH)
            os.mkdir(LOG_PATH)
            os.mkdir(RES_PATH)
            os.mkdir(SRC_PATH)
            copyfile(APP_PATH_ORIG, APP_PATH_DEST)
            copyfile(MANIFEST_PATH_ORIG, MANIFEST_PATH_DEST)
            copyfile(ENTRYPOINT_PATH_ORIG, ENTRYPOINT_PATH_DEST)
        except OSError as exc:
            print(exc)
            return False
        return True

    def create_json_exec_info(self):
        exec_settings = {}
        exec_settings[JSON_EXECINFO_APPID] = self.args.app_id
        exec_settings[JSON_EXECINFO_APPNAME] = self.args.app_name
        exec_settings[JSON_EXECINFO_SIM] = True
        exec_settings[JSON_EXECINFO_DEVMODE] = True
        exec_settings[JSON_EXECINFO_ROBCONN] = {}
        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBID] = ''
        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBIP] = ''
        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBDDSCH] = 0
        exec_settings[JSON_EXECINFO_ROBCONN][JSON_EXECINFO_ROBSERIAL] = ''
        exec_settings[JSON_EXECINFO_LOG] = {}
        exec_settings[JSON_EXECINFO_LOG][JSON_EXECINFO_LOG_FILEENA] = True
        exec_settings[JSON_EXECINFO_LOG][JSON_EXECINFO_LOG_FOLDER] = 'log'
        try:
            with open(EXECSETTINGS_PATH, 'w', encoding='utf-8') as f:
                json.dump(exec_settings, f, ensure_ascii=False, indent=4)
        except FileNotFoundError:
            log_error(f'Could not write file "{EXECSETTINGS_FILE}".')
            return False
        return True

    def run(self, args, unknownargs):
        self.args = args
        self.unknownargs = unknownargs
        if len(os.listdir(CURRENT_PATH)) != 0:
            log_error('Current directory is not empty.')
            return False
        # Files tree
        log_verbose('Creating application files tree...')
        if not self.create_files_tree():
            log_error(f'Could not create files tree')
            return False
        # Json Manifest

        # Json SDK info
        if not self.create_json_exec_info():
            log_error('Could not json sdk info file')
            return False

        log_verbose('\nCorrect!')
        return True



from setuptools import setup, find_packages


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='rayasdk',
    packages=find_packages(),
    version='1.0.10',
    license='MIT',
    description='Raya SDK - Unlimited Robotics Software Development Kit',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Unlimited Robotics',
    author_email='camilo@unlimited-robotics.com',
    url='',
    python_requires=">=3.8",
    download_url = '',
    package_data={'':['./template/*']},
    keywords = ['robotics', 'unlimited-robotics', 'gary'],
    install_requires=[
        'tabulate',
        'importlib_metadata',
        'tqdm',
        'docker',
        'progressbar',
        'simple_file_checksum'
    ],
    entry_points={
        'console_scripts': [
            'rayasdk = rayasdk.__main__:main',
        ],
    },
)


import docker
import hashlib
import pathlib
import subprocess
import json
import os
import shutil
import tarfile
import progressbar
import urllib.request
from simple_file_checksum import get_checksum

from rayasdk.constants import *
from rayasdk.logger import log, LogLevels, log_info, log_error, log_verbose
from rayasdk.logger import set_logger_level
from rayasdk import __version__ as RAYA_VERSION


raya_imgs = {}
docker_client: docker.DockerClient = None


class MyProgressBar():
    def __init__(self):
        self.pbar = None

    def __call__(self, block_num, block_size, total_size):
        if not self.pbar:
            self.pbar=progressbar.ProgressBar(maxval=total_size)
            self.pbar.start()

        downloaded = block_num * block_size
        if downloaded < total_size:
            self.pbar.update(downloaded)
        else:
            self.pbar.finish()


def launch_command(bash_command, suffix, output_file='', silent=False, 
                                        working_dir='', volumes=[], ports=[]):
    complete_bash_command = ''
    if working_dir:
        complete_bash_command += f'cd {working_dir} && '
    complete_bash_command += 'source /opt/raya_os/setup_raya.bash && '
    complete_bash_command += f'{bash_command}'
    if platform.system() == 'Linux':
        sh_command = f'/bin/bash -c "{complete_bash_command}"'
    else: #Windows
        sh_command = f'/bin/bash -c \'{complete_bash_command}\''
    docker_command = ''

    if platform.system() == 'Linux':
        docker_command += 'xhost +local:root >> /dev/null && '
        docker_command += 'docker run -it --rm --privileged --network host '
        docker_command += '-v /tmp/.X11-unix:/tmp/.X11-unix:ro '
        docker_command += '-e DISPLAY=$DISPLAY --env QT_X11_NO_MITSHM=1 --tty '
        docker_command += f'--name {RAYAENV_DOCKER_CONTAINERPREFIX}_{suffix} '
    else: #Windows
        docker_command += 'docker run -it --rm --privileged '
        docker_command += '-v /tmp/.X11-unix:/tmp/.X11-unix:ro '
        docker_command += '-e DISPLAY=host.docker.internal:0.0 --tty '
        docker_command += f'--name {RAYAENV_DOCKER_CONTAINERPREFIX}_{suffix} '
    # TODO: Define for MAC

    for volume in volumes:
        docker_command += f'-v \'{volume}\' '

    for port in ports:
        docker_command += f'-p {port} '

    docker_command += f'{RAYAENV_DOCKER_IMGNAME}:{RAYAENV_DOCKER_VERSION} '
    docker_command += f'{sh_command} '

    if silent:
        if output_file:
            docker_command += f'> {str(output_file)}'
        else:
            docker_command += f'> /dev/null'       
    else:
        if output_file:
            docker_command += f'| tee {str(output_file)}'

    if platform.system() == 'Windows':
        docker_command = f'powershell "{docker_command}"'

    # print(docker_command)

    ret = subprocess.call(docker_command, shell=True)


def download_simulator():
    response = input(('Correct simulator version not found, do you want to'
                                                    ' download it? [Y/n]:'))
    if response not in ['Y', 'y', '']:
        print('aborted')
        return
    print('Removing old versions...')
    try:
        shutil.rmtree(SIMS_HOME)
    except FileNotFoundError:
        pass
    print(f'Downloading Ra-Ya SDK v{GARYSIM_VERSION}...')
    SIMS_HOME.mkdir(parents=True, exist_ok=True)
    try:
        urllib.request.urlretrieve(GARYSIM_URL, str(SIM_TARPATH), 
                                                               MyProgressBar())
    except urllib.error.HTTPError:
        log_error('Download error, try again.')
        return False
    log_info(f'Checking downloaded file...')
    try:
        sha256 = get_checksum(SIM_TARPATH, algorithm="SHA256")
    except FileNotFoundError:
        log_error('Download error, try again.')
        return False
    if GARYSIM_SHA256 != sha256:
        log_error('Download error, try again.')
        return False
    log_info(f'Extracting...')
    simtar = tarfile.open(SIM_TARPATH)
    simtar.extractall(SIMS_HOME)
    # log_info(f'Simulator installed, you can run \'rayasdk simulator\' again')
    return True
    

def run_simulator_bridge():
    
    if not SIM_VERSION_FILE.is_file():
        if not download_simulator():
            return
    sim_file = open(SIM_VERSION_FILE, 'r')
    version = sim_file.readlines()[0]
    sim_file.close()
    if GARYSIM_VERSION not in version:
        if not download_simulator():
            return

    p = subprocess.Popen([str(SIM_BINARY)])

    launch_command(
            ('/opt/raya_os/run_bridge_unity.sh'),
            suffix='simbridge',
            working_dir='/root', 
            ports=['10000:10000', '8789-8890:8789-8890', '8000:8000']
        )
    p.kill()
    p.wait()


def launch_app(app_id, domain, args):
    for container in docker_client.containers.list(all=True):
        if container.name == f'{RAYAENV_DOCKER_CONTAINERPREFIX}_{app_id}':
            container.kill()
            try:
                container.remove()
            except:
                pass

    cmd = f'ROS_DOMAIN_ID={domain} python3 __main__.py ' + ' '.join(args)

    launch_command( cmd, 
                    suffix=app_id, 
                    volumes=[f'{os.getcwd()}:/root/ur/app'], 
                    working_dir='/root/ur/app'
                )


def scanner(min_domain=1, max_domain=100, silent=False):

    for container in docker_client.containers.list(all=True):
        if container.name == f'raya_scanner':
            container.kill()
            try:
                container.remove()
            except:
                pass

    detections = {}
    scan_file = URSDK_TEMP_PATH / 'scan_docker_log.txt'
    launch_command(
            bash_command=('echo ... && ros2 run raya_utils scanner ' 
                          f'{min_domain} {max_domain}'), 
            suffix='scanner', 
            output_file=scan_file, 
            silent=silent
        )
    if not silent:
        log_info('\nFinish scanning')

    try:
        file = open(scan_file, 'r')
        lines = file.readlines()
    except UnicodeDecodeError:
        file = open(scan_file, 'r', encoding='utf_16_le')
        lines = file.readlines()

    for line in lines:
        try:
            if platform.system() == 'Linux':
                line_dict = json.loads(line)
            else: #Windows
                line_dict = json.loads(line.replace('\x00', ''))
            detections[line_dict['id']] = {
                'serial': line_dict['serial'],
                'ip-address': line_dict['ip-address'],
                'dds-domain': line_dict['dds-domain'],
            }
        except json.decoder.JSONDecodeError as e:
            pass
    return detections


def kill_all():
    log_info('Killing all the running Ra-Ya containers...')
    for container in docker_client.containers.list(all=True):
        tags = container.image.tags
        if tags:
            name, tag = tags[0].split(':',1)
            if name=='raya':
                if container.status == 'running':
                    log_info(f'Kill: {container}')    
                    container.kill()
                try:
                    container.remove()
                except:
                    pass


def check_container():
    global raya_imgs, docker_client
    # Check docker installation
    try:
        docker_client = docker.from_env()
    except docker.errors.DockerException:
        log_error(('Docker is not installed or the current user doesn\'t have '
                   'permissions to run it. Please check the Ra-YA '
                   'documentation to install it.'))
        return False
    
    # Check raya images
    for img in docker_client.images.list():
        tags = img.tags
        if tags:
            name, tag = tags[0].split(':',1)
            if name==RAYAENV_DOCKER_IMGNAME:
                raya_imgs[tag] = img

    if RAYAENV_DOCKER_VERSION in raya_imgs:
        return True

    log_info(('Docker image '
                  f'\'{RAYAENV_DOCKER_IMGNAME}:{RAYAENV_DOCKER_VERSION}\' '
                  'not found.'))

    if raya_imgs:
        log_info(('Removing old Ra-Ya containers and images...'))
        for container in docker_client.containers.list(all=True):
            tags = container.image.tags
            if tags:
                name, tag = tags[0].split(':',1)
                if name=='raya':
                    if container.status == 'running':
                        container.kill()
                    try:
                        container.remove()
                    except docker.errors.NotFound:
                        pass
        
        for img_tag in raya_imgs:
            docker_client.images.remove(
                    image=f'{RAYAENV_DOCKER_IMGNAME}:{img_tag}'
                )

    log_info(f'Downloading Ra-Ya OS v{RAYAENV_DOCKER_VERSION} (only once)...')
    try:
        urllib.request.urlretrieve(RAYAENV_DOCKER_URL, str(RAYAENV_DOCKER_IMGPATH),
                                MyProgressBar())
    except urllib.error.HTTPError:
        log_error('Download error, try again.')
        return False
    log_info(f'Checking downloaded file...')
    try:
        sha256 = get_checksum(RAYAENV_DOCKER_IMGPATH, algorithm="SHA256")
    except FileNotFoundError:
        log_error('Download error, try again.')
        return False
    if RAYAENV_DOCKER_SHA256 != sha256:
        log_error('Download error, try again.')
        return False
    log_info(f'Creating image...')
    ret = subprocess.call(
            f'docker load -i {str(RAYAENV_DOCKER_IMGPATH)}', 
            shell=True
        )
    if ret != 0:
        log_error('Error creating the image, try again.')
        return False

    return True




from enum import Enum

verbose = False
quiet = False

class LogLevels(Enum):
    VERBOSE = 1
    INFO = 2
    ERROR = 3

def set_logger_level(_verbose, _quiet):
    global verbose, quiet
    verbose = _verbose
    quiet = _quiet

def log(level, msg, end='\n', flush=False):
    if not quiet:
        if level==LogLevels.INFO or (level==LogLevels.VERBOSE and verbose):
            print(msg, end=end, flush=flush)
    if level==LogLevels.ERROR:
        print(msg, end=end, flush=flush)

def log_verbose(msg, end='\n', flush=False):
    log(LogLevels.VERBOSE, msg, end=end, flush=flush)

def log_info(msg, end='\n', flush=False):
    log(LogLevels.INFO, msg, end=end, flush=flush)

def log_error(msg, end='\n', flush=False):
    log(LogLevels.ERROR, 'ERROR: '+msg, end=end, flush=flush)
